import traceback, inspect
from django.apps import apps
from pyforms_web.basewidget import BaseWidget
from confapp import conf

# Used for older python version
try:
    ModuleNotFoundError
except:
    class ModuleNotFoundError(Exception):
        pass


class AppsManager(object):

    def __init__(self):
        self._plugins_list = []
        self.search_4_plugins()

    def append(self, plugin): self._plugins_list.append(plugin)

    @property
    def plugins(self): return self._plugins_list

    
    def menu(self, user=None, menus=None):
        res = []

        for plugin_class in self.plugins:
            if not hasattr(plugin_class, 'ORQUESTRA_MENU'): continue
            if  menus and \
                (
                    not hasattr(plugin_class, 'ORQUESTRA_MENU') or \
                    not plugin_class.ORQUESTRA_MENU in menus
                ): continue

            add = False

            if hasattr(plugin_class, 'has_permissions'):
                if plugin_class.has_permissions(user): 
                    add = True
            else:
                add = True
            
            if add: res.append(plugin_class)

            plugin_class.fullname = '{0}.{1}'.format( plugin_class.__module__,plugin_class.__name__)

        return res



    def export_settings(self, filename):
        out = open(filename, 'w')

        apps = {}
        
        for plugin_class in self.plugins:
            if issubclass(plugin_class, BaseWidget):
                print(plugin_class)
                apps[plugin_class.__name__.lower()] = '{0}.{1}'.format(
                    plugin_class.__module__,
                    plugin_class.__name__
                )

        out.write( "PYFORMS_APPS = "+str(apps) )
        out.close()


    def search_4_plugins(self):
        for app in apps.get_app_configs():
            try:
                apss_modulename = '{0}.apps'.format(app.module.__name__)

                apps_module = __import__( apss_modulename, fromlist=[''] )
                
                for name in dir(apps_module):
                    obj = getattr(apps_module, name)
                    if inspect.isclass(obj) and hasattr(obj, 'LAYOUT_POSITION'):
                        self.append( obj )
                """
                except ModuleNotFoundError:
                    if conf.ORQUESTRA_SHOW_NO_MODULE_EXCEPTION:
                        traceback.print_exc()
                    pass
                """
            except ModuleNotFoundError:
                if conf.ORQUESTRA_SHOW_NO_MODULE_EXCEPTION:
                    traceback.print_exc()
            except:
                traceback.print_exc()
                



from confapp import conf
from pyforms_web.basewidget import BaseWidget, no_columns, segment

from pyforms_web.controls.control_button import ControlButton
from pyforms_web.controls.control_text import ControlText
from pyforms_web.controls.control_textarea import ControlTextArea

from pyforms_web.controls.control_combo import ControlCombo
from pyforms_web.controls.control_checkbox import ControlCheckBox
from pyforms_web.controls.control_datetime import ControlDateTime
from pyforms_web.controls.control_list import ControlList
from pyforms_web.controls.control_label import ControlLabel

class DefaultApp(BaseWidget):

    TITLE = 'Demo app'

    LAYOUT_POSITION = 0

    def __init__(self, *args, **kwargs):
        super(DefaultApp, self).__init__(*args, **kwargs)

        self._css_btn = ControlButton(
            '<i class="icon toggle on" ></i>Toggle css', 
            default=self.__toggle_css_evt,
            label_visible=False
        )
        self._toggle_btn = ControlButton(
            '<i class="icon eye" ></i>Toggle visibility', 
            default=self.__toggle_visibility_evt,
            label_visible=False
        )
        self._copy_btn   = ControlButton(
            '<i class="icon copy outline" ></i>Copy the text',
            default=self.__copy_text_evt,
            label_visible=False
        )
        self._input = ControlText('Type something here and press the copy button', changed_event=self.__input_changed_evt)
        self._text  = ControlTextArea('Result')
        self._combo = ControlCombo(
            'Combo',
            items=[('Item 1', 1),('Item 2', 2),('Item 3', 3)]
        )
        self._check = ControlCheckBox('Check box')
        self._list  = ControlList('List')
        self._label = ControlLabel('Label', default='Use the label for a dynamic text')

        self.formset = [
            no_columns('_toggle_btn','_copy_btn', '_css_btn'),
            ' ',
            '_input',
            '_text',
            {
                'Free text': [
                    'h1:Header 1',
                    'h2:Header 2',
                    'h3:Header 3',
                    'h4:Header 4',
                    'h5:Header 5',
                    'h1-right:Header 1',
                    'h2-right:Header 2',
                    'h3-right:Header 3',
                    'h4-right:Header 4',
                    'h5-right:Header 5',
                    '-',
                    'Free text here',
                    'msg:Message text',
                    'info:Info message',
                    'warning:Warning message',
                    'alert:Alert message'
                ],
                'Segments': [
                    'The next example has a segment',
                    segment(
                        '_combo',
                        '_check',
                        css='secondary'
                    ),
                    '_list',
                    '_label'
                ]
            }
        ]

    def __toggle_css_evt(self):
        if self._css_btn.css == 'basic green':
            self._css_btn.css = 'inverted red'
            self._label.css = 'red'
            self._css_btn.label = '<i class="icon toggle on" ></i>Toggle css'
        else:
            self._css_btn.css = 'basic green'
            self._label.css = 'inverted green'
            self._css_btn.label = '<i class="icon toggle off" ></i>Toggle css'

    def __toggle_visibility_evt(self):

        if self._input.visible:
            self._input.hide()
        else:
            self._input.show()

        if self._text.visible:
            self._text.hide()
        else:
            self._text.show()

        if self._copy_btn.visible:
            self._copy_btn.hide()
        else:
            self._copy_btn.show()

        if self._css_btn.visible:
            self._css_btn.hide()
        else:
            self._css_btn.show()
            

    def __copy_text_evt(self):

        self._text.value = self._input.value


    def __input_changed_evt(self):
        print(self._input.value)



ORQUESTRA_HOME_FUNCTION 	= None

ORQUESTRA_HOME 		   = 0
ORQUESTRA_NEW_TAB 	   = 1
ORQUESTRA_NEW_WINDOW   = 2
ORQUESTRA_APPEND_HOME  = 3
ORQUESTRA_HOME_FULL    = 4
ORQUESTRA_NEW_TAB_FULL = 5
ORQUESTRA_NEW_BIGWINDOW = 6

ORQUESTRA_EXTRA_CSS_FILE = None

ORQUESTRA_REQUIREAUTH = False

ORQUESTRA_SHOW_NO_MODULE_EXCEPTION = False

ORQUESTRA_TITLE = ''
ORQUESTRA_PAGE_TITLE = ''

ORQUESTRA_GOOGLE_ANALYTICS = None


from confapp                        import conf
from django.conf                    import settings
from django.http                    import HttpResponseRedirect
from django.shortcuts               import render_to_response
from orquestra.apps_manager         import AppsManager


def index(request, app_uid=None):
    manager = AppsManager()
    plugins = manager.plugins

    # no plugins are available.
    # it will show the default application
    if len(plugins)==0:
        return render_to_response('default-app.html' )

    if  conf.ORQUESTRA_REQUIREAUTH and \
        not request.user.is_authenticated:
        return HttpResponseRedirect(settings.LOGIN_URL)
        
    
    ##### find the style and javscripts files #################################################
    style_files, javascript_files = [], []
    for plugin in plugins:
        for staticfile in plugin.STATIC_FILES:
            if staticfile.endswith('.css'): style_files.append(staticfile)
            if staticfile.endswith('.js'):  javascript_files.append(staticfile)
    ###########################################################################################

    #### load menus ###########################################################################
    plugins4menus = sorted(manager.menu(request.user), key=lambda x: (x.ORQUESTRA_MENU,len(x.ORQUESTRA_MENU)) )
    menus         = {}
    active_menus  = {}

    running_menu = None

    for plugin_class in plugins4menus:
        menus_options = plugin_class.ORQUESTRA_MENU.split('>')

        # used to check if a menu should be activated or not
        active_menus[menus_options[0]] = True

        # if an application is not running ignore the submenus
        if app_uid is None and len(menus_options)>1: continue
        
        menu            = type('MenuOption', (object,), {})
        menu.menu_place = menus_options[0]
        menu.uid        = plugin_class.UID if hasattr(plugin_class,'UID') else ''
        menu.url        = plugin_class.ORQUESTRA_URL if hasattr(plugin_class,'ORQUESTRA_URL') else '/app/{0}/'.format(menu.uid)
        menu.target     = 'target={0}'.format(plugin_class.ORQUESTRA_TARGET) if hasattr(plugin_class, 'ORQUESTRA_TARGET') else ''
        menu.label      = plugin_class.TITLE if plugin_class.TITLE else plugin_class.__name__.lower()
        menu.order      = plugin_class.ORQUESTRA_MENU_ORDER if hasattr(plugin_class,'ORQUESTRA_MENU_ORDER') else None
        menu.icon       = plugin_class.ORQUESTRA_MENU_ICON if hasattr(plugin_class, 'ORQUESTRA_MENU_ICON') else None
        menu.anchor     = plugin_class.__name__.lower()
        menu.fullname   = plugin_class.fullname # full name of the class
        menu.parent_menu= None
        menu.active     = False
        menu.submenu_active = False
        menu.submenus   = []
        menu.show_submenu = False
        
        # append main menu
        if len(menus_options)==1:
            menus[plugin_class.__name__] = menu
        
        elif len(menus_options)==2:
            parent_menu = menus.get(menus_options[1], None)
            if parent_menu:
                menu.parent_menu = parent_menu
                parent_menu.submenus.append( menu )
                #menu.parent_menu.active = True

        if hasattr(plugin_class, 'UID') and app_uid==plugin_class.UID: 
            running_menu = menu         
            menu.active  = True
            if menu.parent_menu: 
                menu.parent_menu.show_submenu = True
                menu.parent_menu.submenu_active = True
            else:
                menu.show_submenu = True

    ## sort menus and submenus ######################################################################
    menus = sorted(menus.values(), key=lambda x: (x.menu_place, x.order) )
    for menu in menus: menu.submenus = sorted(menu.submenus, key=lambda x: x.order)
    #################################################################################################

    if running_menu is None and len(menus)>0: running_menu = sorted(menus, key=lambda x: x.order )[0]

    context = {'user': request.user}
    context.update({
        'title': conf.ORQUESTRA_PAGE_TITLE,
        'submenu_title': conf.ORQUESTRA_TITLE,
        'menu_plugins': menus,
        'active_menus': list(set(active_menus)),
        'styles_files': style_files,
        'javascript_files': javascript_files,
        'running_menu': running_menu,
        'GOOGLE_ANALYTICS': conf.ORQUESTRA_GOOGLE_ANALYTICS,
        'extra_css_file': conf.ORQUESTRA_EXTRA_CSS_FILE
    })

    return render_to_response('base-authenticated.html', context )




import os; from setuptools import find_packages, setup 

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme: README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
	name='Orquestra',
	version='1.4',
	packages=find_packages(),
	include_package_data=True,
	description='There for pyforms web.',
	long_description=README,
	url='https://github.com/UmSenhorQualquer/orquestra/',
	author='Ricardo Ribeiro',
	author_email='ricardojvr@gmail.com',
	classifiers=[
		'Environment :: Web Environment',
		'Framework :: Django',
		'Framework :: Django :: 1.9',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: BSD License',  # example license
		'Operating System :: OS Independent',
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.4',
		'Programming Language :: Python :: 3.5',
		'Topic :: Internet :: WWW/HTTP',
		'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
	],
    install_requires=['django-allauth'],
	package_data={'orquestra': [
		'static/*.*',
		'static/jquery-ui/*.js',
		'static/jquery-ui/*.css',
		'static/jquery-ui/images/*.png',
		'static/*.css',
		'static/semantic-ui/*.css',
		'static/semantic-ui/*.js',
		'static/semantic-ui/components/*.css',
		'static/semantic-ui/components/*.js',
		'static/semantic-ui/themes/basic/assets/fonts/*.*',
		'static/semantic-ui/themes/default/images/*.png',
		'static/semantic-ui/themes/default/assets/fonts/*.*',
		'templates/*.html',
		'templates/account/*.html',
		'templates/account/snippets/*.html',
		'templates/plugins/*.js',		
		]
	},
)


from allauth.account.adapter import DefaultAccountAdapter
from allauth.exceptions import ImmediateHttpResponse
from allauth.socialaccount.adapter import DefaultSocialAccountAdapter
from django.conf import settings
from django.shortcuts import render_to_response


class OrquestraAccountAdapter(DefaultAccountAdapter):

	def user_signed_up(self, request, user):
		if hasattr( settings, 'ORQUESTRA_ALLOW_DOMAINS_LOGIN') and settings.ORQUESTRA_ALLOW_DOMAINS_LOGIN:
			if user.email.split('@')[1] not in settings.ORQUESTRA_ALLOW_DOMAINS_LOGIN:
				raise ImmediateHttpResponse(render_to_response('error.html'))



class OrquestraSocialAccountAdapter(DefaultSocialAccountAdapter):

	def pre_social_login(self, request, sociallogin):
		if hasattr( settings, 'ORQUESTRA_ALLOW_DOMAINS_LOGIN') and settings.ORQUESTRA_ALLOW_DOMAINS_LOGIN:
			u = sociallogin.user
			email = u.email
			if email.split('@')[1] not in settings.ORQUESTRA_ALLOW_DOMAINS_LOGIN:
				raise ImmediateHttpResponse(render_to_response('error.html'))